from verbose_print.styles import Styles
from verbose_print.style_decorators import PrintStyle
from typing import Optional


def vprint(
        *args,
        verbose: bool = False,
        end: str = "\n",
        underline: bool = False,
        bold: bool = False,
        color: Optional[Styles] = None
):
    """
    Basic verbose printing function.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param underline: [Optional] Boolean to underline the text or not.
    :param bold: [Optional] Boolean to bold the text or not.
    :param color: [Optional] Color to style the text.
    """
    if not (verbose and args):
        return

    out: str = ""
    if bold:
        out = out + Styles.BOLD
    if underline:
        out = out + Styles.UNDERLINE
    if color is not None:
        out = out + str(color)
    l: int = len(args)
    for i in range(l):
        out = out + args[i]
        if i < l-1:
            out += " "

    print(out, end=end)

    if bold or underline or color:
        print(Styles.END)


@PrintStyle.warning
def vwarn(
        *args,
        verbose: bool = False,
        end: str = "\n",
        underline: bool = False,
        bold: bool = False
):
    """
    Warning verbose printing function, prints in yellow.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param underline: [Optional] Boolean to underline the text or not.
    :param bold: [Optional] Boolean to bold the text or not.
    """
    vprint(*args, verbose=verbose, end=end)


@PrintStyle.fail
def vfail(
        *args,
        verbose: bool = False,
        end: str = "\n",
        underline: bool = False,
        bold: bool = False
):
    """
    Warning verbose printing function, prints in red.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param underline: [Optional] Boolean to underline the text or not.
    :param bold: [Optional] Boolean to bold the text or not.
    """
    vprint(*args, verbose=verbose, end=end)


@PrintStyle.blue
def vblue(
        *args,
        verbose: bool = False,
        end: str = "\n",
        underline: bool = False,
        bold: bool = False
):
    """
    Warning verbose printing function, prints in blue.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param underline: [Optional] Boolean to underline the text or not.
    :param bold: [Optional] Boolean to bold the text or not.
    """
    vprint(*args, verbose=verbose, end=end)


@PrintStyle.cyan
def vcyan(
        *args,
        verbose: bool = False,
        end: str = "\n",
        underline: bool = False,
        bold: bool = False
):
    """
    Warning verbose printing function, prints in cyan.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param underline: [Optional] Boolean to underline the text or not.
    :param bold: [Optional] Boolean to bold the text or not.
    """
    vprint(*args, verbose=verbose, end=end)


@PrintStyle.green
def vsuccess(
        *args,
        verbose: bool = False,
        end: str = "\n",
        underline: bool = False,
        bold: bool = False
):
    """
    Warning verbose printing function, prints in green.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param underline: [Optional] Boolean to underline the text or not.
    :param bold: [Optional] Boolean to bold the text or not.
    """
    vprint(*args, verbose=verbose, end=end)


@PrintStyle.bold
def vbold(
        *args,
        verbose: bool = False,
        end: str = "\n",
        color: Optional[Styles] = None
):
    """
    Basic verbose printing function, prints bolded.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param color: [Optional] Color to style the text.
    """
    vprint(*args, verbose=verbose, end=end, color=color)


@PrintStyle.underline
def vunderline(
        *args,
        verbose: bool = False,
        end: str = "\n",
        color: Optional[Styles] = None
):
    """
    Basic verbose printing function, prints underlined.

    :param args: Everything to print out.
    :param verbose: Boolean to toggle printing.
    :param end: [Optional] String to print at the end.
    :param color: [Optional] Color to style the text.
    """
    vprint(*args, verbose=verbose, end=end, color=color)



from verbose_print.styles import Styles


class PrintStyle:
    """ Class to group all the style decorators. """

    @staticmethod
    def warning(fn):
        """
        Functions to add warning color style to anything printed within
        the given function fn.

        :param fn: Function to execute with warning color style.
        """
        def f(*args, **kwargs):
            print(Styles.YELLOW)
            fn(*args, **kwargs)
            print(Styles.END)
        return f

    @staticmethod
    def fail(fn):
        """
        Functions to add fail color style to anything printed within
        the given function fn.

        :param fn: Function to execute with fail color style.
        """
        def f(*args, **kwargs):
            print(Styles.RED)
            fn(*args, **kwargs)
            print(Styles.END)
        return f

    @staticmethod
    def blue(fn):
        """
        Functions to add blue color style to anything printed within
        the given function fn.

        :param fn: Function to execute with blue color style.
        """
        def f(*args, **kwargs):
            print(Styles.BLUE)
            fn(*args, **kwargs)
            print(Styles.END)
        return f

    @staticmethod
    def cyan(fn):
        """
        Functions to add cyan color style to anything printed within
        the given function fn.

        :param fn: Function to execute with cyan color style.
        """
        def f(*args, **kwargs):
            print(Styles.CYAN)
            fn(*args, **kwargs)
            print(Styles.END)
        return f

    @staticmethod
    def green(fn):
        """
        Functions to add green color style to anything printed within
        the given function fn.

        :param fn: Function to execute with green color style.
        """
        def f(*args, **kwargs):
            print(Styles.GREEN)
            fn(*args, **kwargs)
            print(Styles.END)
        return f

    @staticmethod
    def bold(fn):
        """
        Functions to add bold color style to anything printed within
        the given function fn.

        :param fn: Function to execute with bold style.
        """
        def f(*args, **kwargs):
            print(Styles.BOLD)
            fn(*args, **kwargs)
            print(Styles.END)
        return f

    @staticmethod
    def underline(fn):
        """
        Functions to add underline color style to anything printed within
        the given function fn.

        :param fn: Function to execute with underline style.
        """
        def f(*args, **kwargs):
            print(Styles.UNDERLINE)
            fn(*args, **kwargs)
            print(Styles.END)
        return f
